"""
Channels that couple autocomplete_light and cities_light.
"""

import autocomplete_light

from ..models import City, Country

class CityChannelMixin(object):
    """
    Defines city-specific channel traits.

    search_field
        Set to 'search_names'.
    """
    search_field = 'search_names'

    def query_filter(self, results):
        """
        In addition to filter by search_names, filter with request GET
        variables 'country__name' and 'country__pk'.
        """
        results = super(CityChannelMixin, self).query_filter(results)

        country_name = self.request.GET.get('country__name', False)
        if country_name:
            results = results.filter(country__name=country_name)

        country_pk = self.request.GET.get('country__pk', False)
        if country_pk:
            results = results.filter(country__pk=country_pk)

        return results

class CityChannel(CityChannelMixin, autocomplete_light.ChannelBase):
    """
    Basic Channel for city.
    """
    pass



"""
Couple djangorestframework and cities_light.

It defines a urlpatterns variables, with the following urls:

- cities_light_api_city_list
- cities_light_api_city_detail
- cities_light_api_country_list
- cities_light_api_country_detail

If djangorestframework is installed, all you have to do is add this url
include::

    url(r'^cities_light/api/', include('cities_light.contrib.restframework')),

And there you have a nice API.
"""

from django.conf.urls.defaults import patterns, url
from django.core import urlresolvers
from django.db import models

from djangorestframework.views import *
from djangorestframework.mixins import *
from djangorestframework.resources import ModelResource

from ..models import City, Country

class CityResource(ModelResource):
    """
    ModelResource for City.
    """
    model = City

    def country(self, instance):
        """
        Return the country detail API url.
        """
        return urlresolvers.reverse('cities_light_api_country_detail',
            args=(instance.country.pk,))

class CountryResource(ModelResource):
    """
    ModelResource for Country.
    """
    model = Country

class DetailView(InstanceMixin, ReadModelMixin, ModelView):
    """
    Read-only detail view for djangorestframework.
    """
    pass

class LimitListModelView(ListModelView):
    """
    ListModelView that supports a limit GET request argument.
    """

    def get(self, request, *args, **kwargs):
        """
        Limit the results returned by the parent get(), using the limit GET
        argument.
        """
        limit = request.GET.get('limit', None)
        queryset = super(LimitListModelView, self).get(
            request, *args, **kwargs)
        
        if limit:
            return queryset[:limit]
        else:
            return queryset

class CountryListModelView(LimitListModelView):
    """
    ModelResource for Country.
    """

    def get_query_kwargs(self, request, *args, **kwargs):
        """
        Converts 'q' to 'name__icontains', allow filter by continent.
        """
        kwargs = super(ListModelView, self).get_query_kwargs(request, *args,
            **kwargs)

        if 'continent' in request.GET.keys():
            kwargs['continent'] = request.GET['continent']
        if 'q' in request.GET.keys():
            kwargs['name__icontains'] = request.GET['q']
        
        return kwargs

class CityListModelView(LimitListModelView):
    """
    ListModelView for City.
    """

    def get_query_kwargs(self, request, *args, **kwargs):
        """
        Allows filtering by GET request arguments:

        - country_name,
        - country_pk,
        - continent,
        - q, for search_names__icontains
        """
        kwargs = super(ListModelView, self).get_query_kwargs(request, *args,
            **kwargs)

        if 'country_name' in request.GET.keys():
            kwargs['country__name'] = request.GET['country_name']
        if 'country_pk' in request.GET.keys():
            kwargs['country__pk'] = request.GET['country_pk']
        if 'continent' in request.GET.keys():
            kwargs['country__continent'] = request.GET['continent']
        if 'q' in request.GET.keys():
            kwargs['search_names__icontains'] = request.GET['q']
        
        return kwargs

urlpatterns = patterns('',
    url(
        r'^city/$', 
        CityListModelView.as_view(resource=CityResource),
        name='cities_light_api_city_list',
    ),
    url(
        r'^city/(?P<pk>[^/]+)/$', 
        DetailView.as_view(resource=CityResource),
        name='cities_light_api_city_detail',
    ),
    url(
        r'^country/$', 
        CountryListModelView.as_view(resource=CountryResource),
        name='cities_light_api_country_list',
    ),
    url(
        r'^country/(?P<pk>[^/]+)/$', 
        DetailView.as_view(resource=CountryResource),
        name='cities_light_api_country_detail',
    ),
)



"""
Couples djangorestframework and cities_light.
"""

import autocomplete_light

from ..models import City, Country

from autocomplete_light_channels import CityChannelMixin

class ApiChannelMixin(object):
    """
    Defines model_for_source_url for cities_light.contrib.restframework.
    """
    def model_for_source_url(self, url):
        """
        Return the appropriate model for the urls defined by
        cities_light.contrib.restframework.urlpatterns.
        """
        if 'cities_light/city/' in url:
            return City
        elif 'cities_light/country/' in url:
            return Country

class RemoteCityChannel(CityChannelMixin, ApiChannelMixin, 
    autocomplete_light.RemoteChannelBase):
    """
    Remote channel for City that is compatible with
    cities_light.contrib.restframework.
    """

    def get_source_url_data(self, limit):
        """
        Converts country__pk data to country__name, to avoid PK conflict hell.

        The author insists that he learnt this the hard way.
        """
        data = super(RemoteCityChannel, self).get_source_url_data(limit)
        country_pk = data.pop('country__pk')
        data['country_name'] = Country.objects.get(pk=country_pk).name
        return data

class RemoteCountryChannel(ApiChannelMixin, 
    autocomplete_light.RemoteChannelBase):
    """
    Remote channel for Country that is compatible with
    cities_light.contrib.restframework.
    """
    pass



"""
Widgets that couple cities_light and autocomplete_light.
"""

from django import forms

import autocomplete_light

from ..models import City

class CityAutocompleteWidget(forms.MultiWidget):
    """
    Double autocomplete for a City selection form field, requires
    autocomplete_light.

    This widget wraps around a Country and a City autocomplete widget. It sets
    the city widget's bootstrap to 'countrycity' so that the special javascript
    that connects the country and city field be used to instanciate the deck.
    """
    def __init__(self, channel_name, attrs=None, **kwargs):
        widgets = (
            autocomplete_light.AutocompleteWidget('CountryChannel', max_items=1),
            autocomplete_light.AutocompleteWidget(channel_name, bootstrap='countrycity', 
                **kwargs),
        )
        super(CityAutocompleteWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        """
        Given a value from the database, return a value for each sub-widget.
        """
        if value:
            city = City.objects.get(pk=value)
            return [city.country.pk, value]
        return [None, None]
    
    def value_from_datadict(self, data, files, name):
        """
        Given values from the widget, return the value for the database.
        """
        values = super(CityAutocompleteWidget, self).value_from_datadict(data, files, name)
        return values[1]
    
    def _has_changed(self, initial, data):
        # we want multiwidget for rendering, but simple for data
        return forms.Widget._has_changed(self, initial, data)
